/*
 * Copyright 2006-2007 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sf.ezmorph.object;

import net.sf.ezmorph.MorphException;
import net.sf.ezmorph.ObjectMorpher;

/**
 * Morphs to a Class.<br>
 * This morpher is a singleton.
 *
 * @author Andres Almiray <aalmiray@users.sourceforge.net>
 *
 * Fixed by the chameleon project http://chameleon.ow2.org.
 * This class is using a Class.forName which does not work
 * in OSGi environment.
 *
 */
public final class ClassMorpher implements ObjectMorpher
{
   private static final ClassMorpher INSTANCE = new ClassMorpher();

   /**
    * Returns the singleton instance
    */
   public static ClassMorpher getInstance()
   {
      return INSTANCE;
   }

   private ClassMorpher()
   {
   }

   public boolean equals( Object obj )
   {
      return INSTANCE == obj;
   }

   public int hashCode()
   {
      return 42 + getClass().hashCode();
   }

   /**
    * Method added by Clement Escoffier, for OSGi compliancy.
    * Tries to load the given class using:
    * <ol>
    * <li>the Thread context classloader</li>
    * <li>the current classloader</li>
    * <li>Class.forName (won't work in osgi environment)</li>
    * </ol>
    * @param clazz the class to load
    * @return  the loaded Class
    * @throws ClassNotFoundException if the class cannot be found
    */
  private Class findClass(String clazz) throws ClassNotFoundException {

      // Then try with the TCCL
      ClassLoader tccl = Thread.currentThread().getContextClassLoader();
      try {
          return tccl.loadClass(clazz);
      } catch (ClassNotFoundException e) {
          // Next try...
      }

      // Let's also try with the current classloader
      try {
          return this.getClass().getClassLoader().loadClass(clazz);
      } catch (ClassNotFoundException e) {
          // Next try...
      }

      // Finally delegate to Class.forName (won't work on OSGi, but keep the
      // previous old strategy
      return Class.forName(clazz);

  }

   public Object morph( Object value )
   {
      if( value == null ){
         return null;
      }

      if( value instanceof Class ){
         return (Class) value;
      }

      if( "null".equals( value ) ){
         return null;
      }

      try{
          // FIX - Clement Escoffier
          // In OSGi environment, using Class.forName is not recommended,
          // as it delegates to the classpath classloader.
          // We replace this invocation with a more exhaustive classloading
          // strategy
          // Class.forName( value.toString() );
         return findClass( value.toString() );
      }
      catch( Exception e ){
         throw new MorphException( e );
      }
   }

   public Class morphsTo()
   {
      return Class.class;
   }

   public boolean supports( Class clazz )
   {
      return true;
   }
}